-- node tree graph
-- ben@ioxu.com
-- blog.ioxu.com
require "middleclass"
require "background"
require "vector"
require "util"

-- main data
main = {}
	main.bg = background( "background", "resources/bgcheck-01.png" )
	--main.resolution = {900,600}
	main.resolution = {960,600}
	main.view = {}
	main.view.x = 0
	main.view.y = 0
	main.view.zoom = 1
	love.mouse.setVisible(false)
	main.mousePos = vector( love.mouse.getPosition() )
	main.mouseLast = vector(0,0)
	main.mouseDelta = vector(0,0)
	main.mouseZoomPosition = vector(love.graphics.getWidth(), love.graphics.getHeight()) / (main.view.zoom * 2) -- vector(0,0)
	main.mouseLastZoomPosition = main.mouseZoomPosition:clone()
	main.fullscreen = false

function main.view.reset()
	main.view.x = 0
	main.view.y = 0
	main.view.zoom = 1
end

function main.drawMouse()
	love.graphics.setColor( 255,64,64,255 )
	local mp = vector( love.mouse.getPosition() )
	love.graphics.circle("line", mp.x,mp.y,2.5,16)
end
--


function love.update(dt)
	-- mouse position
	main.mousePos.x, main.mousePos.y= love.mouse.getPosition()

	local mpx = love.mouse.getX()
	local mpy = love.mouse.getY()
	main.mouseDelta.x = mpx - main.mouseLast.x
	main.mouseDelta.y = mpy - main.mouseLast.y

	-- mouse
	if love.mouse.isDown("m") then
		--pan
		main.view.x	= main.view.x + (main.mouseDelta.x / main.view.zoom)
		main.view.y	= main.view.y + (main.mouseDelta.y / main.view.zoom)
	end
	if love.mouse.isDown("r") then
		--zoom
		main.view.zoom = main.view.zoom + main.mouseDelta.y * 0.01
		main.view.zoom = math.clamp(main.view.zoom, 0.01, 5)
		--print("mouse zoom", main.view.zoom)
	end
	

	-- mouse position
	main.mouseLast.x = mpx
	main.mouseLast.y = mpy

end

function love.draw()
	-- start camera
	love.graphics.push()
	-- camera transform
	love.graphics.translate( main.mouseZoomPosition.x, main.mouseZoomPosition.y)
	love.graphics.scale( main.view.zoom )
	love.graphics.translate( -main.mouseZoomPosition.x , -main.mouseZoomPosition.y)
	love.graphics.translate( main.view.x, main.view.y )
	
	-- transformed graphics
	main.bg:draw(main.view.x,main.view.y, main.view.zoom)
	
	love.graphics.setColor( 255,255,255,128 )
	love.graphics.print('graphph', 0, 0,0, 1, 1)
	
	love.graphics.circle("line", 100,100,50,32)
	love.graphics.circle("line", 0,0,10,32)
	love.graphics.circle("line", -100,-300,50,32)
	love.graphics.circle("line", -150,-350,20,32)
	love.graphics.circle("line", 250,-450,20,32)
	
	-- end camera
	love.graphics.pop()
	
	-- screen space
	main.drawMouse()
	love.graphics.print('graphph',love.graphics.getWidth()-100, 10,0, 1, 1)
	
	-- reset
	love.graphics.reset()
end

-- main mouse events
function love.mousepressed( x, y, button )
	if button == 'r' then
		print("mousepressed 'r' ", x, y)

		main.mouseZoomPosition.x = x
		main.mouseZoomPosition.y = y
	end
end


-- main key events
function love.keypressed( )
	if love.keyboard.isDown("escape") then
		-- quit on Escape key
		love.event.push('q')
	end
	if love.keyboard.isDown("r") then
		-- reset view
		main.view.reset()
	end	
	if love.keyboard.isDown("tab") then
		-- fullscreen toggle on Tab key
		if main.fullscreen then
			love.graphics.setMode( main.resolution[1], main.resolution[2], false, true, FSAA )
			main.fullscreen = false
		else
			love.graphics.setMode( main.resolution[1], main.resolution[2], true, true, FSAA )
			main.fullscreen = true
		end
	end
end

